@@ -459,7 +459,7 @@ static gboolean mirage_fragment_mdx_read_sector_data (MirageFragmentMdx *self, g
         mirage_stream_seek(self->priv->data_stream, data_offset, G_SEEK_SET, NULL);
         const gsize read_len = mirage_stream_read(self->priv->data_stream, is_zlib ? self->priv->zlib_buffer : self->priv->buffer, to_read, NULL);
 
-        MIRAGE_DEBUG(self, MIRAGE_DEBUG_FRAGMENT, "%s: read %" G_GINT64_MODIFIER "d bytes\n", __debug__, read_len);
+        MIRAGE_DEBUG(self, MIRAGE_DEBUG_FRAGMENT, "%s: read %" G_GSIZE_MODIFIER "d bytes\n", __debug__, read_len);
 
         /* Decrypt */
         if (self->priv->crypt_handle) {
@@ -506,7 +506,7 @@ static gboolean mirage_fragment_mdx_read_sector_data (MirageFragmentMdx *self, g
             const guint64 start_sector_address = sector_group * self->priv->sectors_in_group;
             const guint64 tweak_counter = 1 + start_sector_address * aligned_sector_size / 16;
 
-            MIRAGE_DEBUG(self, MIRAGE_DEBUG_FRAGMENT, "%s: decrypting data with starting sector number %" G_GINT64_MODIFIER "d, aligned data length is %" G_GINT64_MODIFIER "d, aligned sector size is %d, start value of tweak counter is %" G_GINT64_MODIFIER "d\n", __debug__, start_sector_address, aligned_data_len, aligned_sector_size, tweak_counter);
+            MIRAGE_DEBUG(self, MIRAGE_DEBUG_FRAGMENT, "%s: decrypting data with starting sector number %" G_GINT64_MODIFIER "d, aligned data length is %" G_GSIZE_MODIFIER "d, aligned sector size is %d, start value of tweak counter is %" G_GINT64_MODIFIER "d\n", __debug__, start_sector_address, aligned_data_len, aligned_sector_size, tweak_counter);
 
             gboolean succeeded = mdx_crypto_decipher_buffer_lrw(
                 self->priv->crypt_handle,
