@@ -1,6 +1,6 @@
 /*
  *  libMirage: track
- *  Copyright (C) 2006-2014 Rok Mandeljc
+ *  Copyright (C) 2006-2026 Rok Mandeljc
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 
 #include "mirage/config.h"
 #include "mirage/mirage.h"
+#include "mirage/utils-private.h"
 
 #include <glib/gi18n-lib.h>
 
@@ -39,20 +40,20 @@
 
 
 /**********************************************************************\
- *                         Private structure                          *
+ *                  Object and its private structure                  *
 \**********************************************************************/
 struct _MirageTrackPrivate
 {
     /* Layout settings */
     gint track_number; /* Track number */
     gint start_sector; /* Start sector (where pregap starts)... disc/session-relative address */
-    gint length;       /* Length of track (sum of fragments' length) */
+    gint length; /* Length of track (sum of fragments' length) */
 
     gint track_start; /* Track start sector (where index changes to 1)... track-relative address */
 
     /* Track mode and flags */
     gint flags; /* Track flags */
-    MirageSectorType sector_type;  /* Type of sectors that comprise track */
+    MirageSectorType sector_type; /* Type of sectors that comprise track */
 
     gchar *isrc; /* ISRC */
     gboolean isrc_fixed; /* Is ISRC fixed due to one of track's fragments having subchannel? */
@@ -69,6 +70,9 @@ struct _MirageTrackPrivate
 };
 
 
+G_DEFINE_TYPE_WITH_PRIVATE(MirageTrack, mirage_track, MIRAGE_TYPE_OBJECT)
+
+
 /**********************************************************************\
  *                          Private functions                         *
 \**********************************************************************/
@@ -82,8 +86,8 @@ static gchar *mirage_track_scan_for_isrc (MirageTrack *self)
     }
 
     /* According to INF8090, ISRC, if present, must be encoded in at least
-       one sector in 100 consequtive sectors. So we read first hundred
-       sectors' subchannel, and extract ISRC if we find it. */
+     * one sector in 100 consequtive sectors. So we read first hundred
+     * sectors' subchannel, and extract ISRC if we find it. */
     gint start_address = mirage_fragment_get_address(fragment);
 
     g_object_unref(fragment);
@@ -131,9 +135,9 @@ static void mirage_track_rearrange_indices (MirageTrack *self)
 {
     /* Rearrange indices: set their numbers */
     /* Indices numbers start with 2 (0 and 1 are controlled via
-       get/set_track_start... and while we're at it, if index lies before
-       track start, remove it (it most likely means track start got changed
-       after indices were added) */
+     * get/set_track_start... and while we're at it, if index lies before
+     * track start, remove it (it most likely means track start got changed
+     * after indices were added) */
     gint cur_index = 2;
     g_assert(self->priv->indices_list != NULL);
     MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: rearranging indices (%d indices found)...\n", __debug__, g_list_length(self->priv->indices_list));
@@ -156,7 +160,7 @@ static void mirage_track_rearrange_indices (MirageTrack *self)
 static void mirage_track_commit_topdown_change (MirageTrack *self)
 {
     /* No need to rearrange indices, because they don't have anything to do with
-       the global layout */
+     * the global layout */
 
     /* Rearrange fragments: set start sectors */
     gint cur_fragment_address = 0;
@@ -214,7 +218,7 @@ static void mirage_track_fragment_layout_changed_handler (MirageTrack *self, Mir
 static void mirage_track_remove_fragment (MirageTrack *self, MirageFragment *fragment)
 {
     /* Disconnect signal handler (find it by handler function and user data) */
-    g_signal_handlers_disconnect_by_func(fragment, mirage_track_fragment_layout_changed_handler, self);
+    mirage_signal_handlers_disconnect_by_func(fragment, G_CALLBACK(mirage_track_fragment_layout_changed_handler), self);
 
     /* Remove fragment from list and unref it */
     self->priv->fragments_list = g_list_remove(self->priv->fragments_list, fragment);
@@ -368,7 +372,7 @@ void mirage_track_set_ctl (MirageTrack *self, gint ctl)
     if (ctl & 0x02) flags |= MIRAGE_TRACK_FLAG_COPYPERMITTED;
     if (ctl & 0x08) flags |= MIRAGE_TRACK_FLAG_FOURCHANNEL;
 
-    return mirage_track_set_flags(self, flags);
+    mirage_track_set_flags(self, flags);
 }
 
 /**
@@ -414,8 +418,8 @@ gint mirage_track_get_ctl (MirageTrack *self)
 void mirage_track_set_isrc (MirageTrack *self, const gchar *isrc)
 {
     /* ISRC is encoded in track's subchannel. This means that if subchannel is
-       provided by one of track's fragments (and therefore won't be generated by
-       libMirage), ISRC shouldn't be settable... */
+     * provided by one of track's fragments (and therefore won't be generated by
+     * libMirage), ISRC shouldn't be settable... */
 
     if (self->priv->isrc_fixed) {
         MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: ISRC is already encoded in subchannel!\n", __debug__);
@@ -454,7 +458,7 @@ const gchar *mirage_track_get_isrc (MirageTrack *self)
  * @self: a #MirageTrack
  * @address: (in): sector address
  * @abs: (in): absolute address
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves a sector. @address is sector address for which a #MirageSector
  * object representing sector should be returned. @abs specifies whether @address
@@ -505,7 +509,7 @@ MirageSector *mirage_track_get_sector (MirageTrack *self, gint address, gboolean
     /* Fragments work with fragment-relative addresses, so get fragment's start address */
     fragment_start = mirage_fragment_get_address(fragment);
 
-    MIRAGE_DEBUG(self, MIRAGE_DEBUG_SECTOR, "%s: got fragment %p for track-relative address 0x%X; fragment relative address: 0x%X\n", __debug__, fragment, address, address - fragment_start);
+    MIRAGE_DEBUG(self, MIRAGE_DEBUG_SECTOR, "%s: got fragment %p for track-relative address 0x%X; fragment relative address: 0x%X\n", __debug__, (void *)fragment, address, address - fragment_start);
 
     /* Main channel data */
     if (!mirage_fragment_read_main_data(fragment, relative_address - fragment_start, &main_buffer, &main_length, &local_error)) {
@@ -529,7 +533,7 @@ MirageSector *mirage_track_get_sector (MirageTrack *self, gint address, gboolean
     mirage_object_set_parent(MIRAGE_OBJECT(sector), self);
 
     /* Feed data to sector; fragment's reading code guarantees that
-       subchannel format is PW96 */
+     * subchannel format is PW96 */
     if (!mirage_sector_feed_data(sector, absolute_address, self->priv->sector_type, main_buffer, main_length, MIRAGE_SUBCHANNEL_PW, subchannel_buffer, subchannel_length, 0, &local_error)) {
         g_set_error(error, MIRAGE_ERROR, MIRAGE_ERROR_TRACK_ERROR, Q_("Failed to feed data: %s"), local_error->message);
         g_error_free(local_error);
@@ -551,7 +555,7 @@ MirageSector *mirage_track_get_sector (MirageTrack *self, gint address, gboolean
  * mirage_track_put_sector:
  * @self: a #MirageTrack
  * @sector: (in): a #MirageSector representing sector to be written
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Writes the @sector to track. The address at which sector is written
  * is retrieved from sector's property; for this function to succeed,
@@ -574,16 +578,16 @@ gboolean mirage_track_put_sector (MirageTrack *self, MirageSector *sector, GErro
     gint main_length, subchannel_length;
 
     /* Note that we check only if relative_address is greater than track's
-       length. This accounts for the case when sector's address is one
-       more than track's length, in which case we will append the sector
-       to track */
+     * length. This accounts for the case when sector's address is one
+     * more than track's length, in which case we will append the sector
+     * to track */
     if (relative_address < 0 || relative_address > self->priv->length) {
         g_set_error(error, MIRAGE_ERROR, MIRAGE_ERROR_TRACK_ERROR, Q_("Sector address out of range!"));
         return FALSE;
     }
 
     /* We allow appending of sector to the track only if relative address
-       matches track's length, and if there is no track following this one */
+     * matches track's length, and if there is no track following this one */
     if (relative_address == self->priv->length) {
         MirageTrack *next_track = mirage_track_get_next(self, NULL);
         if (next_track) {
@@ -615,10 +619,10 @@ gboolean mirage_track_put_sector (MirageTrack *self, MirageSector *sector, GErro
     /* Fragments work with fragment-relative addresses, so get fragment's start address */
     fragment_start = mirage_fragment_get_address(fragment);
 
-    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: got fragment %p for track-relative address 0x%X; fragment relative address: 0x%X\n", __debug__, fragment, relative_address, relative_address - fragment_start);
+    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: got fragment %p for track-relative address 0x%X; fragment relative address: 0x%X\n", __debug__, (void *)fragment, relative_address, relative_address - fragment_start);
 
     /* Get expected main channel data size from the fragment; if fragment
-       expects subchannel, we always feed 96-byte raw interleaved PW */
+     * expects subchannel, we always feed 96-byte raw interleaved PW */
     main_length = mirage_fragment_main_data_get_size(fragment);
     subchannel_length = mirage_fragment_subchannel_data_get_size(fragment);
 
@@ -815,7 +819,7 @@ void mirage_track_add_fragment (MirageTrack *self, gint index, MirageFragment *f
 {
     gint num_fragments;
 
-    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: (index: %i, fragment: %p)\n", __debug__, index, fragment);
+    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: %s (index: %i, fragment: %p)\n", __debug__, __func__, index, (void *)fragment);
 
     /* First fragment, last fragment... allow negative indexes to go from behind */
     num_fragments = mirage_track_get_number_of_fragments(self);
@@ -842,19 +846,19 @@ void mirage_track_add_fragment (MirageTrack *self, gint index, MirageFragment *f
     self->priv->fragments_list = g_list_insert(self->priv->fragments_list, fragment, index);
 
     /* Connect fragment modified signal */
-    g_signal_connect_swapped(fragment, "layout-changed", (GCallback)mirage_track_fragment_layout_changed_handler, self);
+    g_signal_connect_swapped(fragment, "layout-changed", G_CALLBACK(mirage_track_fragment_layout_changed_handler), self);
 
     /* Bottom-up change */
     mirage_track_commit_bottomup_change(self);
 
-    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: end\n", __debug__);
+    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: %s: end\n", __debug__, __func__);
 }
 
 /**
  * mirage_track_remove_fragment_by_index:
  * @self: a #MirageTrack
  * @index: (in): index of fragment to be removed.
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Removes fragment from track.
  *
@@ -905,7 +909,7 @@ void mirage_track_remove_fragment_by_object (MirageTrack *self, MirageFragment *
  * mirage_track_get_fragment_by_index:
  * @self: a #MirageTrack
  * @index: (in): index of fragment to be retrieved
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves fragment by index. If @index is negative, fragments from the end of
  * track are retrieved (e.g. -1 is for last track, -2 for second-to-last
@@ -945,7 +949,7 @@ MirageFragment *mirage_track_get_fragment_by_index (MirageTrack *self, gint inde
  * mirage_track_get_fragment_by_address:
  * @self: a #MirageTrack
  * @address: (in): address belonging to fragment to be retrieved
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves fragment by address. @address must be valid (track-relative) sector
  * address that is part of the fragment to be retrieved (i.e. lying between fragment's
@@ -983,8 +987,8 @@ MirageFragment *mirage_track_get_fragment_by_address (MirageTrack *self, gint ad
 /**
  * mirage_track_enumerate_fragments:
  * @self: a #MirageTrack
- * @func: (in) (scope call): callback function
- * @user_data: (in) (closure): data to be passed to callback function
+ * @func: (in) (scope call) (closure user_data): callback function
+ * @user_data: (in) (nullable): data to be passed to callback function
  *
  * Iterates over fragments list, calling @func for each fragment in the layout.
  *
@@ -1008,7 +1012,7 @@ gboolean mirage_track_enumerate_fragments (MirageTrack *self, MirageEnumFragment
 /**
  * mirage_track_find_fragment_with_subchannel:
  * @self: a #MirageTrack
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves first fragment that contains subchannel data. A reference to fragment
  * is stored in @fragment; it should be released with g_object_unref() when no
@@ -1098,7 +1102,7 @@ gint mirage_track_get_number_of_indices (MirageTrack *self)
  * mirage_track_add_index:
  * @self: a #MirageTrack
  * @address: (in): address at which the index is to be added
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Adds index to track.
  *
@@ -1113,7 +1117,7 @@ gint mirage_track_get_number_of_indices (MirageTrack *self)
 gboolean mirage_track_add_index (MirageTrack *self, gint address, GError **error)
 {
     MirageIndex *index;
-    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: address: 0x%X\n", __debug__, address);
+    MIRAGE_DEBUG(self, MIRAGE_DEBUG_TRACK, "%s: %s (address: 0x%X)\n", __debug__, __func__, address);
 
     /* Make sure we're not trying to put index before track start (which has index 1) */
     if (address < self->priv->track_start) {
@@ -1141,7 +1145,7 @@ gboolean mirage_track_add_index (MirageTrack *self, gint address, GError **error
  * mirage_track_remove_index_by_number:
  * @self: a #MirageTrack
  * @number: (in): index number of index to be removed
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Removes index from track. This causes index numbers of remaining indices to be readjusted.
  *
@@ -1183,7 +1187,7 @@ void mirage_track_remove_index_by_object (MirageTrack *self, MirageIndex *index)
  * mirage_track_get_index_by_number:
  * @self: a #MirageTrack
  * @number: (in): index number of index to be retrieved
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves index by index number. If @number is negative, indices from the end of
  * track are retrieved (e.g. -1 is for index, -2 for second-to-last index, etc.).
@@ -1222,7 +1226,7 @@ MirageIndex *mirage_track_get_index_by_number (MirageTrack *self, gint number, G
  * mirage_track_get_index_by_address:
  * @self: a #MirageTrack
  * @address: (in): address belonging to index to be retrieved
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves index by address. @address must be valid (track-relative) sector
  * address that is part of the index to be retrieved (i.e. lying between index's
@@ -1260,8 +1264,8 @@ MirageIndex *mirage_track_get_index_by_address (MirageTrack *self, gint address,
 /**
  * mirage_track_enumerate_indices:
  * @self: a #MirageTrack
- * @func: (in) (scope call): callback function
- * @user_data: (in) (closure): user data to be passed to callback function
+ * @func: (in) (scope call) (closure user_data): callback function
+ * @user_data: (in) (nullable): user data to be passed to callback function
  *
  * Iterates over indices list, calling @func for each index.
  *
@@ -1301,8 +1305,8 @@ gint mirage_track_get_number_of_languages (MirageTrack *self)
  * mirage_track_add_language:
  * @self: a #MirageTrack
  * @code: (in): language code for the added language
- * @language: (in) (transfer full) (allow-none): a #MirageLanguage to be added
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @language: (in) (transfer full): a #MirageLanguage to be added
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Adds language to track.
  *
@@ -1340,7 +1344,7 @@ gboolean mirage_track_add_language (MirageTrack *self, gint code, MirageLanguage
  * mirage_track_remove_language_by_index:
  * @self: a #MirageTrack
  * @index: (in): index of language to be removed
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Removes language from track.
  *
@@ -1369,7 +1373,7 @@ gboolean mirage_track_remove_language_by_index (MirageTrack *self, gint index, G
  * mirage_track_remove_language_by_code:
  * @self: a #MirageTrack
  * @code: (in): language code of language to be removed
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Removes language from track.
  *
@@ -1410,7 +1414,7 @@ void mirage_track_remove_language_by_object (MirageTrack *self, MirageLanguage *
  * mirage_track_get_language_by_index:
  * @self: a #MirageTrack
  * @index: (in): index of language to be retrieved
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves language by index. If @index is negative, languages from the end of
  * track are retrieved (e.g. -1 is for last language, -2 for second-to-last
@@ -1450,7 +1454,7 @@ MirageLanguage *mirage_track_get_language_by_index (MirageTrack *self, gint inde
  * mirage_track_get_language_by_code:
  * @self: a #MirageTrack
  * @code: (in): language code of language to be retrieved
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves language by language code.
  *
@@ -1486,8 +1490,8 @@ MirageLanguage *mirage_track_get_language_by_code (MirageTrack *self, gint code,
 /**
  * mirage_track_enumerate_languages:
  * @self: a #MirageTrack
- * @func: (in) (scope call): callback function
- * @user_data: (in) (closure): data to be passed to callback function
+ * @func: (in) (scope call) (closure user_data): callback function
+ * @user_data: (in) (nullable): data to be passed to callback function
  *
  * Iterates over languages list, calling @func for each language.
  *
@@ -1511,7 +1515,7 @@ gboolean mirage_track_enumerate_languages (MirageTrack *self, MirageEnumLanguage
 /**
  * mirage_track_get_prev:
  * @self: a #MirageTrack
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves track that is placed before @self in session layout.
  *
@@ -1540,7 +1544,7 @@ MirageTrack *mirage_track_get_prev (MirageTrack *self, GError **error)
 /**
  * mirage_track_get_next:
  * @self: a #MirageTrack
- * @error: (out) (allow-none): location to store error, or %NULL
+ * @error: (out) (optional): location to store error, or %NULL
  *
  * Retrieves track that is placed after @self in session layout
  *
@@ -1570,9 +1574,6 @@ MirageTrack *mirage_track_get_next (MirageTrack *self, GError **error)
 /**********************************************************************\
  *                             Object init                            *
 \**********************************************************************/
-G_DEFINE_TYPE_WITH_PRIVATE(MirageTrack, mirage_track, MIRAGE_TYPE_OBJECT)
-
-
 static void mirage_track_init (MirageTrack *self)
 {
     self->priv = mirage_track_get_instance_private(self);
@@ -1597,7 +1598,7 @@ static void mirage_track_dispose (GObject *gobject)
         if (entry->data) {
             MirageFragment *fragment = entry->data;
             /* Disconnect signal handler and unref */
-            g_signal_handlers_disconnect_by_func(fragment, mirage_track_fragment_layout_changed_handler, self);
+            mirage_signal_handlers_disconnect_by_func(fragment, G_CALLBACK(mirage_track_fragment_layout_changed_handler), self);
             g_object_unref(fragment);
 
             entry->data = NULL;
@@ -1625,7 +1626,7 @@ static void mirage_track_dispose (GObject *gobject)
     }
 
     /* Chain up to the parent class */
-    return G_OBJECT_CLASS(mirage_track_parent_class)->dispose(gobject);
+    G_OBJECT_CLASS(mirage_track_parent_class)->dispose(gobject);
 }
 
 static void mirage_track_finalize (GObject *gobject)
@@ -1639,7 +1640,7 @@ static void mirage_track_finalize (GObject *gobject)
     g_free(self->priv->isrc);
 
     /* Chain up to the parent class */
-    return G_OBJECT_CLASS(mirage_track_parent_class)->finalize(gobject);
+    G_OBJECT_CLASS(mirage_track_parent_class)->finalize(gobject);
 }
 
 static void mirage_track_class_init (MirageTrackClass *klass)
